# -*- coding: utf-8 -*-
import os
# 本页面用于多个py文件之间共享全局变量
global ui
pool = 0
pool2 = 0
gui=0
csvfile=''
tdinit=False
mdinit=False
totaltasknum = 0
finishtasknum = 0
DialogBackTestPoint=0
BackTestThreadPoint=0
#存储合约组的交易的合约
list_instrumentgroupenable = [{},{},{},{},{},{},{},{},{},{},{}]
#屏蔽第一次K线更新
notfirstupdateklineUi=False
#存储Talib公式条件
dict_talibcondition = {}
#设置Talib窗口打开状态
DialogTalibState=False
#设置合约的窗口打开状态
DialogSetInstrumentState=False
#过滤规则窗口打开状态
DialogSetRuleState =False
dict_position = {}
dict_order = {}
dict_trader = {}
# 只有 data_kline_M1  (M1周期数据)是从服务器获取或实时生成
# 其他周期K线数据是通过Python的Pandas本地合成
# 本地生成数据包括 data_kline_M3、data_kline_M5、data_kline_M10、data_kline_M15、data_kline_M30、data_kline_M60、data_kline_M120、data_kline_D1
# 字段分别为 ID,Data,Time,Open,Close,Low,High
# 调用方式如下
# print("data_kline_M1: " + str(data_kline_M1))
# print("data_kline_M1: " + str(data_kline_M1[0]))
# print("data_kline_M1: " + str(data_kline_M1[1]))
# print("data_kline_M1: " + str(data_kline_M1[2]))
# print("data_kline_M11: " + str(data_kline_M1[2][1]))
# print("data_kline_M12: " + str(data_kline_M1[2][2]))
# 保存订阅的合约的数据， 取值方法 dict_data_kline_M1['ag2110']
dict_dataframe_kline_M1 = {}
dict_dataframe_kline_M3 = {}
dict_dataframe_kline_M5 = {}
dict_dataframe_kline_M10 = {}
dict_dataframe_kline_M15 = {}
dict_dataframe_kline_M30 = {}
dict_dataframe_kline_M60 = {}
dict_dataframe_kline_M120 = {}
dict_dataframe_kline_D1 = {}
# 显示的K线图的个周期数据
data_kline_M1 = []
data_kline_M3 = []
data_kline_M5 = []
data_kline_M10 = []
data_kline_M15 = []
data_kline_M30 = []
data_kline_M60 = []
data_kline_M120 = []
data_kline_D1 = []
global ticklist
ticklist = []
# VNPY官方Plus服务账号
global Plus_UserName, Plus_Password
Plus_UserName = "admin"
Plus_Password = "000000"
# 认证用户权限
global PlusAuthState
# K线数据模式，0实时TICK生成K线，1从服务器补齐当日K线，2从服务器补齐多日K线（需Plus会员）
global klineserverstate
# 用于保存选择框变量
global list_INE, list_CFFEX, list_SHFE, list_DCE, list_CZCE
global dict_exchange, dict_instrument
global tradestate, thistoday,backteststate
# 保存策略执行的合约编码
global dict_strategyinstrument
# 保存当前pygraph K线图、闪电图选中选中合约对应的交易所，合约编码，周期
global selectperiod, selectexchange, selectinstrumenid
# 实例化交易库和行情库作为全局遍历保存
global td, md, rc, vk, currpath
currpath = os.path.abspath(os.path.dirname(__file__))
td = 0
md = 0
rc = 0
vk = 0
selectinstrumenid = '1'
selectperiod = 1


def _init():  # 初始化
    global PlusAuthState
    PlusAuthState = 0
    global klineserverstate
    klineserverstate = 0
    global tradestate, thistoday,backteststate
    thistoday = 0
    tradestate = 0
    backteststate = 0
    global td, md, rc, vk, currpath
    global list_INE, list_CFFEX, list_SHFE, list_DCE, list_CZCE, selectperiod, selectexchange, selectinstrumenid
    global dict_strategyinstrument
    list_INE = []
    list_CFFEX = []
    list_SHFE = []
    list_DCE = []
    list_CZCE = []
    global dict_exchange, dict_instrument
    dict_exchange = {}
    dict_instrument = {}
    dict_strategyinstrument = {}
    global _global_dict
    _global_dict = {}


def caompareinstrumwent(InstrumentID):
    global selectinstrumenid
    # print(selectinstrumenid+', '+InstrumentID)
    if selectinstrumenid == InstrumentID:
        return True
    else:
        return False


def set_value(key, value):
    """ 定义一个全局变量 """
    _global_dict[key] = value


def get_value(key, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return _global_dict[key]
    except:
        pass
        # KeyError


def set_list_INE(value):
    """ 定义一个全局变量 """
    # list_INE[id] = value
    list_INE.append(value)


def get_list_INE(id, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return list_INE[id]
    except:
        pass
        # KeyError


def getlen_list_INE():
    return len(list_INE)


def set_list_CFFEX(value):
    """ 定义一个全局变量 """
    list_CFFEX.append(value)


def get_list_CFFEX(id, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return list_CFFEX[id]
    except:
        pass
        # KeyError


def getlen_list_CFFEX():
    return len(list_CFFEX)


def set_list_SHFE(value):
    """ 定义一个全局变量 """
    list_SHFE.append(value)


def get_list_SHFE(id, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return list_SHFE[id]
    except:
        pass
        # KeyError


def getlen_list_SHFE():
    return len(list_SHFE)


def set_list_DCE(value):
    """ 定义一个全局变量 """
    list_DCE.append(value)


def get_list_DCE(id, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return list_DCE[id]
    except:
        pass
        # KeyError


def getlen_list_DCE():
    return len(list_DCE)


def set_list_CZCE(value):
    """ 定义一个全局变量 """
    list_CZCE.append(value)


def get_list_CZCE(id, defValue=None):
    """ 获得一个全局变量,不存在则返回默认值 """
    try:
        return list_CZCE[id]
    except:
        pass
        # KeyError


def getlen_list_CZCE():
    return len(list_CZCE)


TRADINGDATE = 0
KLINETIME = 1
INSTRUMENT = 2
OPEN = 3
CLOSE = 4
LOW = 5
HIGH = 6
VOL = 7



# -*- coding=utf-8 -*-
# 官方网站：http://www.vnpy.cn

import time
from CTPMarketType import *
from globalType import *
from ctypes import *
import os.path
import globalvar

global Index

from PyQt5.QtWidgets import QMessageBox

Index = {}


class CTPMarket(object):
    def __init__(self):
        #self.signal_md_tick = signal_md_tick
        currpath = os.path.abspath(os.path.dirname(__file__))
        #self.vnmd = CDLL(currpath + '\\vnctpmd.dll')
        self.vnmd = CDLL('vnctpmd.dll')
        global Index

        '''
        i = 0
        while (i < 5):
            time.sleep(1)
            if (self.vnmd.IsInitOK() == 0):
                i += 1
            else:
                break
        else:
            print('market init error')
            return
        '''
        self.fReqUserLogin = self.vnmd.ReqUserLogin
        self.fReqUserLogin.argtypes = []
        self.fReqUserLogin.restype = c_int32

        self.fReqUserLogout = self.vnmd.ReqUserLogout
        self.fReqUserLogout.argtypes = []
        self.fReqUserLogout.restype = c_int32

        self.fUnSubscribeMarketData = self.vnmd.UnSubscribeMarketData
        self.fUnSubscribeMarketData.argtypes = [c_char_p]
        self.fUnSubscribeMarketData.restype = c_int32

        self.fGetApiVersion = self.vnmd.GetApiVersion
        self.fGetApiVersion.argtypes = []
        self.fGetApiVersion.restype = c_char_p

        self.fGetTradingDay = self.vnmd.GetTradingDay
        self.fGetTradingDay.argtypes = []
        self.fGetTradingDay.restype = c_char_p

        self.fRegisterFront = self.vnmd.RegisterFront
        self.fRegisterFront.argtypes = [c_char_p]

        self.fRegisterNameServer = self.vnmd.RegisterNameServer
        self.fRegisterNameServer.argtypes = [c_char_p]

        self.fSetPrintState = self.vnmd.SetPrintState
        self.fSetPrintState.argtypes = [c_bool]

        self.fGetKlineData = self.vnmd.GetKlineData
        self.fGetKlineData.argtypes = [c_int32]
        self.fGetKlineData.restype = c_void_p

        self.fOpenLog = self.vnmd.OpenLog
        self.fOpenLog.argtypes = []

        self.fCloseLog = self.vnmd.CloseLog
        self.fCloseLog.argtypes = []

        self.fSubscribeMarketData = self.vnmd.SubscribeMarketData
        self.fSubscribeMarketData.argtypes = [c_void_p]
        self.fSubscribeMarketData.restype = c_int32

        self.fSubscribeForQuoteRsp = self.vnmd.SubscribeForQuoteRsp
        self.fSubscribeForQuoteRsp.argtypes = [c_char_p]

        self.InstrumentNum = self.vnmd.GetInstrumentNum()

        self.fInitMD = self.vnmd.InitMD
        self.fInitMD.argtypes = []
        self.fInitMD.restype = c_int32

        self.fLog = self.vnmd.Log
        self.fLog.argtypes = [c_char_p, c_char_p]
        self.fLog.restype = c_void_p

        self.fSetRejectdataTime = self.vnmd.SetRejectdataTime
        self.fSetRejectdataTime.argtypes = [c_double, c_double, c_double, c_double, c_double, c_double, c_double,
                                            c_double]
        self.fSetRejectdataTime.restype = c_void_p

        self.fGetKline = self.vnmd.GetKline
        self.fGetKline.argtypes = [c_char_p, c_int32]
        self.fGetKline.restype = c_void_p

        # self.Index = {}#dict()
        '''
        self.InstrumentNum = self.vnmd.GetInstrumentNum()
        self.KlineMaxLen = self.vnmd.GetKlineMaxLen()
        print('InstrumentNum  : ' + str(self.InstrumentNum) + ',' + str(self.KlineMaxLen))

        for i in range(self.InstrumentNum):
            for j in range(self.KlineMaxLen):
                klinedata = self.fGetKlineData(i, j)
                klinedata = cast(klinedata, POINTER(KLineDataType))
                # self.Index[str(data[0].InstrumentID)] = data
                self.Index[str(klinedata[0].InstrumentID)] = klinedata
        pass
        '''

        '''
        self.Index = dict()
        for i in range(self.InstrumentNum):
            data = self.fGetData(i)
            #data = cast(data, POINTER(sDepMarketData))
            self.Index[str(data[0].InstrumentID.encode('gb2312'))] = data
        pass
        '''

        '''
        self.InstrumentNum = self.vnmd.GetInstrumentNum()
        self.Index = dict()
        for i in range(self.InstrumentNum):
            data = self.fGetData(i)
            data = cast(data, POINTER(VNDepMarketData))
            self.Index[str(data[0].InstrumentID)] = data
        pass
        '''

    '''
    def TradingDay(self, InstrumentID):
        # 交易日
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].TradingDay
        else:
            return ''

    def InstrumentID(self, InstrumentID):
        # 合约代码
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].InstrumentID
        else:
            return ''

    def ExchangeID(self, InstrumentID):
        # 交易所代码
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].ExchangeID
        else:
            return ''

    def ExchangeInstID(self, InstrumentID):
        # 合约在交易所的代码
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].ExchangeInstID
        else:
            return ''

    def LastPrice(self, InstrumentID):
        # 最新价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].LastPrice, 1)
        else:
            return -1

    def PreSettlementPrice(self, InstrumentID):
        # 上次结算价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].PreSettlementPrice, 1)
        else:
            return -1

    def PreClosePrice(self, InstrumentID):
        # 昨收盘
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].PreClosePrice, 1)
        else:
            return -1

    def PreOpenInterest(self, InstrumentID):
        # 昨持仓量
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].PreOpenInterest
        else:
            return -1

    def OpenPrice(self, InstrumentID):
        # 今开盘
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].OpenPrice, 1)
        else:
            return -1

    def highestprice(self, InstrumentID):
        # 最高价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].HighestPrice, 1)
        else:
            return -1

    def LowestPrice(self, InstrumentID):
        # 最低价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].LowestPrice, 1)
        else:
            return -1

    def Volume(self, InstrumentID):
        # 数量
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].Volume
        else:
            return -1

    def Turnover(self, InstrumentID):
        # 成交金额
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].Turnover
        else:
            return -1

    def OpenInterest(self, InstrumentID):
        # 持仓量
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].OpenInterest
        else:
            return -1

    def ClosePrice(self, InstrumentID):
        # 今收盘
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].ClosePrice, 1)
        else:
            return -1

    def SettlementPrice(self, InstrumentID):
        # 本次结算价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].SettlementPrice, 1)
        else:
            return -1

    def UpperLimitPrice(self, InstrumentID):
        # 涨停板价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].UpperLimitPrice, 1)
        else:
            return -1

    def LowerLimitPrice(self, InstrumentID):
        # 跌停板价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].LowerLimitPrice, 1)
        else:
            return -1

    def PreDelta(self, InstrumentID):
        # 昨虚实度
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].PreDelta
        else:
            return -1

    def CurrDelta(self, InstrumentID):
        # 今虚实度
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].CurrDelta
        else:
            return -1

    def UpdateTime(self, InstrumentID):
        # 最后修改时间
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].UpdateTime
        else:
            return ''

    def UpdateMillisec(self, InstrumentID):
        # 最后修改毫秒
        if InstrumentID.encode('gb2312') in self.Index:
            return self.Index[InstrumentID.encode('gb2312')][0].UpdateMillisec
        else:
            return 0

    def BidPrice1(self, InstrumentID):
        # 申买价一
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidPrice1, 1)
        else:
            return -1

    def BidVolume1(self, InstrumentID):
        # 申买量一
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidVolume1, 1)
        else:
            return -1

    def AskPrice1(self, InstrumentID):
        # 申卖价一
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskPrice1, 1)
        else:
            return -1

    def AskVolume1(self, InstrumentID):
        # 申卖量一
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskVolume1, 1)
        else:
            return -1

    def BidPrice2(self, InstrumentID):
        # 申买价二
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidPrice2, 1)
        else:
            return -1

    def BidVolume2(self, InstrumentID):
        # 申买量二
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidVolume2, 1)
        else:
            return None

    def AskPrice2(self, InstrumentID):
        # 申卖价二
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskPrice2, 1)
        else:
            return None

    def AskVolume2(self, InstrumentID):
        # 申卖量二
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskVolume2, 1)
        else:
            return None

    def BidPrice3(self, InstrumentID):
        # 申买价三
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidPrice3, 1)
        else:
            return None

    def BidVolume3(self, InstrumentID):
        # 申买量三
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidVolume3, 1)
        else:
            return None

    def AskPrice3(self, InstrumentID):
        # 申卖价三
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskPrice3, 1)
        else:
            return None

    def AskVolume3(self, InstrumentID):
        # 申卖量三
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskVolume3, 1)
        else:
            return None

    def BidPrice4(self, InstrumentID):
        # 申买价四
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidPrice4, 1)
        else:
            return None

    def BidVolume4(self, InstrumentID):
        # 申买量四
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidVolume4, 1)
        else:
            return None

    def AskPrice4(self, InstrumentID):
        # 申卖价四
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskPrice4, 1)
        else:
            return None

    def AskVolume4(self, InstrumentID):
        # 申卖量四
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskVolume4, 1)
        else:
            return None

    def BidPrice5(self, InstrumentID):
        # 申买价五
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidPrice5, 1)
        else:
            return None

    def BidVolume5(self, InstrumentID):
        # 申买量五
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].BidVolume5, 1)
        else:
            return None

    def AskPrice5(self, InstrumentID):
        # 申卖价五
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AskPrice5, 1)
        else:
            return None

    def AskVolume5(self, InstrumentID):
        # 申卖量五
        if InstrumentID.encode('gb2312') in self.Index:
            return (self.Index[InstrumentID.encode('gb2312')][0].AskVolume5, 1)
        else:
            return None

    def AveragePrice(self, InstrumentID):
        # 当日均价
        if InstrumentID.encode('gb2312') in self.Index:
            return round(self.Index[InstrumentID.encode('gb2312')][0].AveragePrice, 1)
        else:
            return None
    '''

    '''
    def UpdateGetKlineData(self):
        self.InstrumentNum = self.vnmd.GetInstrumentNum()
        self.Index = dict()
        for i in range(self.InstrumentNum):
            data = self.fGetKlineData(i)
            data = cast(data, POINTER(KLineDataType))
            self.Index[str(data[0].InstrumentID)] = data
        pass
    '''

    def GetKline(self, InstrumentID, ID):
        try:
            data = self.fGetKline(InstrumentID.encode("utf-8"),ID)
            return cast(data, POINTER(KLineDataType))
        except Exception as e:
            print("GetKline Error:" + repr(e))

    def GetKline2(self, InstrumentID, ID):
        try:
            print('BBB: '+InstrumentID)

            data = self.fGetKline(InstrumentID.encode("utf-8"), ID)
            # data = self.fGetKline(InstrumentID,ID)
            return cast(data, POINTER(KLineDataType))
        except Exception as e:
            print("GetKline Error:" + repr(e))

    def UpdateGetKlineData(self, InstrumentID):
        linelist = []
        '''
        for k in iter(linelist):
            self.Index[InstrumentID][k] = k
        
        self.Index[InstrumentID]=linelist
        for j in range(self.KlineMaxLen):
            klinedata = self.fGetKlineData(len(self.Index),j)
            klinedata = cast(klinedata, POINTER(KLineDataType))
            #self.Index[InstrumentID][j] = klinedata
            self.Index[InstrumentID].append(klinedata)

        '''

    def KlineM1_InstrumentID(self, InstrumentID, ID):
        print('len: ' + str(len(self.Index)))
        # 交易日
        if InstrumentID in self.Index:
            print('python KlineM1_InstrumentID' + self.Index[InstrumentID][ID].InstrumentID +
                  self.Index[InstrumentID][ID][0].InstrumentID)

            return self.Index[InstrumentID][ID].InstrumentID
        else:
            return ''

    def KlineM1_OpenPrice(self, InstrumentID, ID):
        # 交易日
        if InstrumentID in self.Index:
            return self.Index[InstrumentID][ID].OpenPrice

    # 订阅询价
    def SubscribeForQuoteRsp(self, InstrumentID):
        # 订阅合约时，请注意合约的大小写，中金所和郑州交易所是大写，上海和大连期货交易所是小写的
        self.fSubscribeForQuoteRsp(c_char_p(InstrumentID.encode('gb2312')))
        # data = self.fGetData(len(self.Index))
        # data = cast(data, POINTER(sDepMarketData))
        # self.Index[InstrumentID] = data

    def SubscribeMarketData(self, InstrumentID):
        # 订阅合约时，请注意合约的大小写，中金所和郑州交易所是大写，上海和大连期货交易所是小写的
        try:
            thisInstrumentID = VNInstrument()
            thisInstrumentID.InstrumentID = bytes(InstrumentID, encoding="utf-8")
            result = self.fSubscribeMarketData(byref(thisInstrumentID))
            self.UpdateGetKlineData(InstrumentID)
        except Exception as e:
            globalvar.vnfa.AsynSleep(2000)
            self.SubscribeMarketData(InstrumentID)
        return result

    def UnSubscribeMarketData(self, a):
        # 订阅合约时，请注意合约的大小写，中金所和郑州交易所是大写，上海和大连期货交易所是小写的
        thisInstrumentID = VNInstrument()
        thisInstrumentID.InstrumentID = bytes(a, encoding="utf-8")
        return self.fUnSubscribeMarketData(byref(thisInstrumentID))

    def Log(self, content):
        return self.fLog(content.encode('gb2312'))

    def SetRejectdataTime(self, begintime1, endtime1, begintime2, endtime2, begintime3, endtime3, begintime4, endtime4):
        return self.fSetRejectdataTime(begintime1, endtime1, begintime2, endtime2, begintime3, endtime3, begintime4,
                                       endtime4)

    def ReqUserLogin(self):
        return self.fReqUserLogin()

    def ReqUserLogout(self):
        return self.fReqUserLogout()

    def UnSubscribeMarketData(self, InstrumentID):
        return self.fUnSubscribeMarketData(InstrumentID.encode('gb2312'))

    def SetPrintState(self, printstate):
        return self.fSetPrintState(printstate)

    def GetApiVersion(self):
        return self.fGetApiVersion()

    def GetTradingDay(self):
        return self.fGetTradingDay()

    def RegisterFront(self, pszFrontAddress):
        self.fRegisterFront(pszFrontAddress.encode('gb2312'))

    def RegisterNameServer(self, pszNsAddress):
        self.fRegisterNameServer(pszNsAddress.encode('gb2312'))

    # 建立连接回调
    def OnFrontConnected(self):
        pass

    # 断开连接回调
    def OnFrontDisconnected(self, a):
        pass

    # 登录回调
    def OnRspUserLogin(self, a):
        pass

    # 退出登录回调
    def OnRspUserLogout(self, a):
        pass

    # 行情回调
    def OnRtnDepthMarketData(self, a):
        pass

    # 合约订阅合约回调
    def OnRspSubMarketData(self, a):
        pass

    # 合约反订阅合约回调
    def OnRspUnSubMarketData(self, a):
        pass

    # 注册Python的OnFrontConnected回调函数指针，对应CTP c++的OnFrontConnected方法
    def VNRegOnFrontConnected(self):
        CMPFUNC = CFUNCTYPE(None)
        self.vnmd.VNRegOnFrontConnected(CMPFUNC(self.OnFrontConnected))

    # 注册Python的OnFrontDisconnected回调函数指针，对应CTP c++的OnFrontDisconnected方法
    def VNRegOnFrontDisconnected(self):
        CMPFUNC = CFUNCTYPE(None, c_void_p)
        self.vnmd.VNRegOnFrontDisconnected(CMPFUNC(self.OnFrontDisconnected))

    # 注册Python的OnRspUserLogin回调函数指针，对应CTP c++的OnRspUserLogin方法
    def VNRegOnRspUserLogin(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNCThostFtdcRspUserLoginField))
        self.vnmd.VNRegOnRspUserLogin(CMPFUNC(self.OnRspUserLogin))

    # 注册Python的OnRspUserLogout回调函数指针，对应CTP c++的OnRspUserLogout方法
    def VNRegOnRspUserLogout(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNCThostFtdcRspUserLoginField))
        self.vnmd.VNRegOnRspUserLogout(CMPFUNC(self.OnRspUserLogout))

    # 注册Python的OnRspSubMarketData回调函数指针，对应CTP c++的OnRspSubMarketData方法
    def VNRegOnRspSubMarketData(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNDepMarketData))
        self.vnmd.VNRegOnRspSubMarketData(CMPFUNC(self.OnRspSubMarketData))

    # 注册行情回调
    def VNRegOnRtnDepthMarketData(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNDepMarketData))
        self.vnmd.VNRegOnRtnDepthMarketData(CMPFUNC(self.OnRtnDepthMarketData))

    # 注册订阅合约回调
    def VNRegOnRspSubMarketData(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNInstrument))
        self.vnmd.VNRegOnRspSubMarketData(CMPFUNC(self.OnRspSubMarketData))

    # 注册反订阅合约回调
    def VNRegOnRspUnSubMarketData(self):
        CMPFUNC = CFUNCTYPE(None, POINTER(VNInstrument))
        self.vnmd.VNRegOnRspUnSubMarketData(CMPFUNC(self.OnRspUnSubMarketData))

    # 注册策略管理
    def VNRegStrategyManager(self):
        pass

    def InitMD(self):
        return self.fInitMD()

    def OpenLog(self):
        self.fOpenLog()

    def CloseLog(self):
        self.fCloseLog()

    # 策略管理
    # 合约反订阅合约回调
    def OnStrategyCalculate(self, a):
        pass

    # 注册反订阅合约回调
    def VNRegOnStrategyCalculate(self, strategyfile):
        currpath = os.path.abspath(os.path.dirname(__file__))
        self.strategyfile = strategyfile
        print('构造策略类：' + self.strategyfile)
        CMPFUNC = CFUNCTYPE(None, POINTER(VNStrategy))
        self.vnmd.VNRegOnStrategyCalculate(CMPFUNC(self.OnStrategyCalculate))

    # 打开策略窗口
    def OpenStrategyProcessWindow(self):
        self.vnmd.OpenStrategyProcessWindow()

    # 关闭策略窗口
    def CloseStrategyProcessWindow(self):
        self.vnmd.CloseStrategyProcessWindow()

    # --------------------------策略管理器----------------------
    '''
    # 注册C++策略管理器的1个进程的句柄
    def VNRegOnStrategyCalculate(self,hwnd):
        self.vnmd.VNRegOnStrategyCalculate222(hwnd)
    '''
    # --------------------------策略管理器----------------------



# -*- coding=utf-8 -*-
from ctypes import *
from globalType import *
# 官方网站：http://www.vnpy.cn
class KLineDataType(Structure):
    _fields_ = [('Open', c_double),
                ('High', c_double),
                ('Low', c_double),
                ('Close', c_double),
                ('KlineTime', c_double),
                ('Volume', c_int),
                ('Minutes', c_int),
                ('InstrumentID', c_char * 81),
                ('TradingDay', c_char * 9)
                ]


class TThostFtdcInstrumentIDType(Structure):
    _fields_ = [('value', c_char * 81)]
    pass


class VN_char11(Structure):
    _fields_ = [('value', c_char * 11)]
    pass


class VNLoginInfo(Structure):
    _fields_ = [('brokeid', c_char * 9),
                ('investor', c_char * 31),
                ('password', c_char * 31),
                ('appid', c_char * 31),
                ('authcode', c_char * 31),
                ('productInfo', c_char * 31),
                ('adder1', c_char * 50),
                ('adder2', c_char * 50),
                ('adder3', c_char * 50)

                ]
    pass


class VNDepMarketData(Structure):
    _fields_ = [('TradingDay', c_char * 9),
                ('reserve1', c_char * 31),
                ('ExchangeID', c_char * 9),
                ('reserve2', c_char * 31),
                ('LastPrice', c_double),
                ('PreSettlementPrice', c_double),
                ('PreClosePrice', c_double),
                ('PreOpenInterest', c_double),
                ('OpenPrice', c_double),
                ('HighestPrice', c_double),
                ('LowestPrice', c_double),
                ('Volume', c_int),
                ('Turnover', c_double),
                ('OpenInterest', c_double),
                ('ClosePrice', c_double),
                ('SettlementPrice', c_double),
                ('UpperLimitPrice', c_double),
                ('LowerLimitPrice', c_double),
                ('PreDelta', c_double),
                ('CurrDelta', c_double),
                ('UpdateTime', c_char * 9),
                ('UpdateMillisec', c_int32),
                ('BidPrice1', c_double),
                ('BidVolume1', c_int32),
                ('AskPrice1', c_double),
                ('AskVolume1', c_int32),
                ('BidPrice2', c_double),
                ('BidVolume2', c_int32),
                ('AskPrice2', c_double),
                ('AskVolume2', c_int32),
                ('BidPrice3', c_double),
                ('BidVolume3', c_int32),
                ('AskPrice3', c_double),
                ('AskVolume3', c_int32),
                ('BidPrice4', c_double),
                ('BidVolume4', c_int32),
                ('AskPrice4', c_double),
                ('AskVolume4', c_int32),
                ('BidPrice5', c_double),
                ('BidVolume5', c_int32),
                ('AskPrice5', c_double),
                ('AskVolume5', c_int32),
                ('AveragePrice', c_double),
                ('ActionDay',  c_char * 9),
                ('InstrumentID',  c_char * 81),
                ('ExchangeInstID', c_char * 81),
                ('BandingUpperPrice', c_double),
                ('BandingLowerPrice', c_double)
                ]
    pass


class VNCThostFtdcRspUserLoginField(Structure):
    _fields_ = [('TradingDay', c_char * 9),  # 交易日
                ('LoginTime', c_char * 9),  # 登录成功时间
                ('BrokerID', c_char * 11),  # 经纪公司代码
                ('UserID', c_char * 16),  # 用户代码
                ('SystemName', c_char * 41),  # 交易系统名称
                ('FrontID', c_int32),  # 前置编号
                ('SessionID', c_int32),  # 会话编号
                ('MaxOrderRef', c_char * 13),  # 最大报单引用
                ('SHFETime', c_char * 9),  # 上期所时间
                ('DCETime', c_char * 9),  # 大商所时间
                ('CZCETime', c_char * 9),  # 郑商所时间
                ('FFEXTime', c_char * 9),  # 中金所时间
                ('INETime', c_char * 9)  # 能源中心时间
                ]
    pass



class VN_CThostFtdcForQuoteRspField(Structure):
    _fields_ = [('TradingDay', c_char * 9),  # 交易日
                ('InstrumentID', c_char * 81),  # 合约代码
                ('ForQuoteSysID', c_char * 21),  # 询价编号
                ('ForQuoteTime', c_char * 9),  # 询价时间
                ('ActionDay', c_char * 9),  # 业务日期
                ('ExchangeID', c_char * 9)  # 交易所代码
                ]
    pass


NULL = 100


# 周期定义
VN_ALL = 10000  # 所有周期
VN_M1 = 10001  # M1   1分钟
VN_M3 = 10002  # M3   3分钟
VN_M5 = 10003  # M5   5分钟
VN_M10 = 10004  # M10  10分钟
VN_M15 = 10005  # M15  15分钟
VN_M30 = 10006  # M30  30分钟
VN_M60 = 10007  # M60  60分钟
VN_M120 = 10008  # M120 120分钟
VN_D1 = 10009  # D1   1日

# PRICE TYPE
VN_CLOSE = 10010  # c_char('0')	# 收盘价
VN_OPEN = 10011  # c_char('1')	# 开盘价
VN_HIGH = 10012  # c_char('2')	# 最高价
VN_LOW = 10013  # c_char('3')	# 最低价



# 官方网站：http://www.kucps.com
import threading
import logging
import os
import sys
import configparser
# make the example runnable without the need to install
from os.path import abspath, dirname
sys.path.insert(0, abspath(dirname(abspath(__file__)) + '/..'))
curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(os.path.split(rootPath)[0])
print(sys.path)

# CTP行情库
from CTPMarket import *
# CTP交易库
# 导入时间库
import time
from threading import Thread

# MyCTPMarket类继承自CTPMarket类
class MyCTPMarket(CTPMarket):
    global list_INE,list_CFFEX,list_SHFE,list_DCE,list_CZCE
    global dict_exchange, dict_instrument
    def __init__(self):
        super().__init__()

    def UpdateMainType(self, instrumentMain, instrumentName, exchange, jump):
        global tempdate
        returnvalue = 0
        savedate = time.strftime("%Y%m%d", time.localtime())
        tvs = (int(float(savedate) * 0.000001)) * 1000000
        tempdate = int((float(savedate) - float(tvs)) * 0.01)
        tj = 0
        for j in range(12):
            if exchange == 'INE':
                returnvalue = mounthyear4(tempdate, j)
                update_instrument(instrumentMain + str(returnvalue), instrumentName, exchange, jump)
                globalvar.md.SubscribeMarketData(instrumentMain + str(returnvalue))
                if j==0:
                    if len(instrumentMain) > 0:
                        globalvar.set_list_INE(instrumentMain+','+instrumentName+','+exchange)
                globalvar.dict_exchange[instrumentMain + str(returnvalue)] = exchange+',能源所'
                globalvar.dict_instrument[instrumentMain + str(returnvalue)] = instrumentMain+','+instrumentName+','+exchange
            elif exchange == 'CFFEX':
                returnvalue = mounthyear4(tempdate, j)
                update_instrument(instrumentMain + str(returnvalue), instrumentName, exchange, jump)
                globalvar.md.SubscribeMarketData(instrumentMain + str(returnvalue))
                if j==0:
                    if len(instrumentMain) > 0:
                        globalvar.set_list_CFFEX(instrumentMain+','+instrumentName+','+exchange)
                globalvar.dict_exchange[instrumentMain + str(returnvalue)] = exchange+',中金所'
                globalvar.dict_instrument[instrumentMain + str(returnvalue)] = instrumentMain+','+instrumentName+','+exchange
            elif exchange == 'SHFE':
                returnvalue = mounthyear4(tempdate, j)
                update_instrument(instrumentMain + str(returnvalue), instrumentName, exchange, jump)
                globalvar.md.SubscribeMarketData(instrumentMain + str(returnvalue))
                if j==0:
                    if len(instrumentMain) > 0:
                        globalvar.set_list_SHFE(instrumentMain+','+instrumentName+','+exchange)
                globalvar.dict_exchange[instrumentMain + str(returnvalue)] = exchange+',上期所'
                globalvar.dict_instrument[instrumentMain + str(returnvalue)] = instrumentMain+','+instrumentName+','+exchange
            elif exchange == 'DCE':
                returnvalue = mounthyear4(tempdate, j)
                update_instrument(instrumentMain + str(returnvalue), instrumentName, exchange, jump)
                globalvar.md.SubscribeMarketData(instrumentMain + str(returnvalue))
                if j==0:
                    if len(instrumentMain) > 0:
                        globalvar.set_list_DCE(instrumentMain+','+instrumentName+','+exchange)
                globalvar.dict_exchange[instrumentMain + str(returnvalue)] = exchange+',大商所'
                globalvar.dict_instrument[instrumentMain + str(returnvalue)] = instrumentMain+','+instrumentName+','+exchange
            elif exchange == 'CZCE':
                returnvalue = mounthyear3(tempdate, j)
                update_instrument(instrumentMain + str(returnvalue), instrumentName, exchange, jump)
                globalvar.md.SubscribeMarketData(instrumentMain + str(returnvalue))
                if j==0:
                    if len(instrumentMain) > 0:
                        globalvar.set_list_CZCE(instrumentMain+','+instrumentName+','+exchange)
                globalvar.dict_exchange[instrumentMain + str(returnvalue)] = exchange+',郑商所'
                globalvar.dict_instrument[instrumentMain + str(returnvalue)] = instrumentMain+','+instrumentName+','+exchange
        return returnvalue

    def mackinstrumentID(self):
        instrumentidlist = []
        with open('InstrumentID.ini', 'r') as f:
            for line in f:
                instrumentIDarr = line.strip('\n').split(',')
                instrumentidlist.append(list(line.strip('\n').split(',')))
                self.UpdateMainType(instrumentIDarr[1], instrumentIDarr[2], instrumentIDarr[3], instrumentIDarr[4])

        #globalvar.printlist()
        #print("输出list: " + globalvar.list_INE[-1])
        #ui.callback_md_combox()

    def updatehistorystockagain(self,instrument,historystocklist2):
        tempinstrument=instrument
        if  tempinstrument.strip('0123456789')==instrument:
            return
        reagain=0
        if instrument in historystocklist2:
            for i in range(len(historystocklist2)):
                print(historystocklist2 )
                if str(historystocklist2[i])==instrument:
                    historystocklist2.pop(i)
                    reagain=1
                    break
        if reagain==1:
            self.updatehistorystockagain(instrument,historystocklist2)
        else:
            historystocklist2.insert(0,instrument)
            hstr=''
            for i in range(len(historystocklist2)):
                if i < len(historystocklist2) - 1:
                    hstr = hstr + historystocklist2[i] + ','
                else:
                    hstr = hstr + historystocklist2[i]
            with open("historystock.ini", "w") as f:
                f.write(hstr)


    def updatehistorystock(self,instrument):
        tempinstrument=instrument
        if  tempinstrument.strip('0123456789')==instrument:
            return
        reagain=0
        with open('historystock.ini', 'r') as f:
            for line in f:
                historystocklist = line.strip('\n').split(',')
                if instrument in historystocklist:
                    for i in range(len(historystocklist)):
                        if str(historystocklist[i])==instrument:
                            historystocklist.pop(i)
                            reagain=1
                            break
        if reagain==1:
            self.updatehistorystockagain(instrument,historystocklist)
        else:
            historystocklist.insert(0,instrument)
            hstr=''
            for i in range(len(historystocklist)):
                if i<len(historystocklist)-1:
                    hstr=hstr+historystocklist[i]+','
                else:
                    hstr = hstr + historystocklist[i]
            with open("historystock.ini", "w") as f:
                f.write(hstr)



    def ReqUserLogin(self):
        pass
    '''
    #重载
    def KlineM1 (self):
        print("wdg")
        super().KlineM1()
    '''


    def OnRtnDepthMarketData(self, tickdata):
        print("OnRtnDepthMarketData")
        print(str(tickdata[0].InstrumentID, encoding="utf-8")+' , ' +str(tickdata[0].UpdateTime)+' , ' +
              str(tickdata[0].UpdateMillisec)+' , ' +str(tickdata[0].LastPrice) )

    # 合约订阅回调
    def OnRspSubMarketData(self, a):
        print(u'订阅合约成功OnRspSubMarketData'+a.contents.InstrumentID)
        log_todaymd('订阅合约成功OnRspSubMarketData'+a.contents.InstrumentID)

    # 合约订阅回调
    def OnRspUnSubMarketData(self, a):
        print(u'反订阅合约成功OnRspUnSubMarketData'+a.contents.InstrumentID)
        log_todaymd('反订阅合约成功OnRspUnSubMarketData'+a.contents.InstrumentID)

    # 登录回调
    def OnRspUserLogin(self, a):
        # print(a.contents.a1, a.contents.a2)
        print(u'行情登录成功OnRspUserLogin')
        log_todaymd('行情登录成功OnRspUserLogin')
        self.SubscribeMarketData('rb2205')
        #self.mackinstrumentID()
        # 订阅品种zn1610，接收Tick数据,不根据Tick生成其他周期价格数据,但可根据AddPeriod函数添加周期价格数据的设置

    # 退出登录回调
    def OnRspUserLogout(self, a):
        # print(a.contents.a1, a.contents.a2)
        print(u'行情登出成功OnRspUserLogout')
        log_todaymd('行情登出成功OnRspUserLogout')

    # 建立连接回调
    def OnFrontConnected(self):
        print("连接行情服务器成功OnFrontConnected")
        log_todaymd('连接行情服务器成功OnFrontConnected')

    # 断开连接回调
    def OnFrontDisconnected(self, a):
        print("断开与行情服务器连接OnFrontDisconnected")
        log_todaymd('断开与行情服务器连接OnFrontDisconnected')

    def SubscribeMarketData(self, a):
        return CTPMarket().SubscribeMarketData(a)

    def InitMD(self):
        return CTPMarket().InitMD()

    def OpenLog(self):
        return CTPMarket().OpenLog()

    def CloseLog(self):
        return CTPMarket().CloseLog()

    def ReqUserLogin(self):
        return CTPMarket().ReqUserLogin()

    def ReqUserLogout(self):
        return CTPMarket().ReqUserLogout()

class RegTdThreadOnFrontConnected(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnFrontConnected()

class RegTdThreadOnFrontDisconnected(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnFrontDisconnected()

class RegTdThreadOnRspUserLogin(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRspUserLogin()

class RegTdThreadOnRspUserLogout(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRspUserLogout()

class RegTdThreadOnRspQryInvestorPosition(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRspQryInvestorPosition()


class RegTdThreadOnRspQryTradingAccount(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRspQryTradingAccount()


class RegTdThreadOnRtnOrder(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRtnOrder()

class RegTdThreadOnRtnTrade(Thread):
    def __init__(self, name, td):
        super().__init__()
        self.name = name
        self.td = td
    def run(self):
        self.td.VNRegOnRtnTrade()

# ---------------------------------------
class RegMdThreadOnFrontConnected(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnFrontConnected()

class RegMdThreadOnFrontDisconnected(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnFrontDisconnected()

# RegThreadTdOnRspUserLogin
class RegMdThreadOnRspUserLogin(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnRspUserLogin()


# RegThreadTdOnRspUserLogin
class RegMdThreadOnRspUserLogout(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnRspUserLogout()

class RegMdThreadOnRtnDepthMarketData(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnRtnDepthMarketData()

class RegMdThreadOnRspSubMarketData(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnRspSubMarketData()

class RegMdThreadOnRspUnSubMarketData(Thread):
    def __init__(self, name, md):
        super().__init__()
        self.name = name
        self.md = md
    def run(self):
        self.md.VNRegOnRspUnSubMarketData()




instrumenttableid=0
def update_instrument(instrumentID,instrumentName,exchange,jump):
    pass



import os
# https://www.cnblogs.com/AmyHu/p/10654500.html
# https://blog.csdn.net/edward_zcl/article/details/88809212


import importlib
def dynamic_import(module):
    return importlib.import_module(module)


import globalvar
globalvar._init()


def log_todaymd(mystr):
    print(mystr)

global tempdata
thisdate = 0
global savedate
def mounthyear4(thisdate,add):
    year = (int)(thisdate*0.01)
    mounth =  thisdate - year * 100
    mounth = mounth+add
    if mounth > 12:
        mounth = mounth-12
        year = year + 1
    thisdate = year * 100 + mounth
    return thisdate

def mounthyear3(thisdate,add):
    year = (int)(thisdate*0.01)
    y= (int)(thisdate*0.001)
    mounth =  thisdate - year * 100
    mounth = mounth+add
    if mounth > 12:
        mounth = mounth-12
        year = year + 1
    thisdate = year * 100 + mounth
    thisdate=thisdate-1000*y
    return thisdate

def function_md(md):
    #注册异步回调函数线程（多线程回调，可绕过GIL锁）
    RegMdThreadOnFrontConnected('OnFrontConnected', md).start()
    RegMdThreadOnFrontDisconnected('OnFrontDisconnected', md).start()
    RegMdThreadOnRspUserLogin('OnRspUserLogin', md).start()
    RegMdThreadOnRspUserLogout('OnRspUserLogout', md).start()
    RegMdThreadOnRtnDepthMarketData('OnRtnDepthMarketData', md).start()
    RegMdThreadOnRspSubMarketData('OnRspSubMarketData', md).start()
    RegMdThreadOnRspUnSubMarketData('OnRspUnSubMarketData', md).start()

    time.sleep(1)
    result= md.InitMD()
    print("mdresult:"+str(result))

    if result == 0:
        log_todaymd('行情接口配置文件读取成功')
    elif result == 1:
        log_todaymd('行情接口配置文件读取错误，请检查vnctpmd.ini是否存在')
        return
    elif result == 2:
        log_todaymd('行情接口配置文件读取错误，请检查vnctpmd.ini配置文件至少要配置一个行情IP地址')
        return

    #mackinstrumentID()
    # md.RegisterFront()
    # Login()，不需要参数，Login读取QuickLibTD.ini的配置信息，并登录
    # 返回0， 表示登录成功，
    # 返回1， FutureTDAccount.ini错误
    # 返回2， 登录超时
    # 第一次是自动登录，退出登录后，如需再登陆，需添加以下登陆代码
    # 调用交易接口元素，通过 “ 接口变量.元素（接口类内部定义的方法或变量） ” 形式调用
    # Login()，不需要参数，Login读取QuickLibTD.ini的配置信息，并登录
    # 返回0， 表示登录成功，
    # 返回1， FutureTDAccount.ini错误
    # 返回2， 登录超时
    # print ('login: ', retLogin)

    # if md.Login() == 0:
    if md.ReqUserLogin() == 0:
        log_todaymd('发送登录行情服务器请求成功')
    else:
        log_todaymd('发送登录行情服务器请求失败')
    while 1:
        #会一直循环执行，可在这个循环内需增加策略判断，达到下单条件即可下单
        time.sleep(30)  # 系统休眠

def getctpdata():
    class MDThread(Thread):
        def __init__(self, tname):
            super(MDThread, self).__init__()
            self.tname = tname
        def run(self):
            time.sleep(3)
            globalvar.md = MyCTPMarket()
            function_md(globalvar.md)
    tm = MDThread('tm')
    tm.start()


def main():
    getctpdata()


if __name__ == "__main__":
    main()



import setuptools
 
requirements = ['configparser']       # 自定义工具中需要的依赖包
 
setuptools.setup(
    name="vnapi",       # 自定义工具包的名字
    version="1.0",             # 版本号
    author="上海量贝信息科技有限公司",           # 作者名字
    author_email="public@vnpy.cn",  # 作者邮箱
    description="Quantitative software", # 自定义工具包的简介
    license='GPLV3',           # 许可协议
    url="http://www.vnpy.cn",              # 项目开源地址
    packages=setuptools.find_packages(),  # 自动发现自定义工具包中的所有包和子包
    install_requires=requirements,  # 安装自定义工具包需要依赖的包
    python_requires='>=3.5'         # 自定义工具包对于python版本的要求
)


# -*- coding=utf-8 -*-
from ctypes import *
VN_FEE_CONSTANT = 0  # 固定手续费
VN_FEE_RATE = 1  # 按比例收取手续费
THOST_FTDC_D_Buy   = '0'   #买
THOST_FTDC_D_Sell  = '1'   #卖
THOST_FTDC_OF_Open = '0'  # 开仓
THOST_FTDC_OF_Close = '1'  # 平仓
THOST_FTDC_OF_CloseToday = 3  # 平今仓
TradeType_ACCOUNT         = 1  #报单给柜台的交易(实盘，simnow仿真等柜台交易)
TradeType_VIRTUALACCOUNT  = 2  #回测用本地虚拟账号



class VNDEFTradingAccountField(Structure):
    _fields_ = [('BrokerID', c_char * 11),  # 经纪公司代码
                ('InvestorID', c_char * 13),  # 投资者代码
                ('Prebalance', c_double),  # 合约代码
                ('Current', c_double),  # 报单引用
                ('Available', c_double),  # 用户代码
                ('Rate', c_double),  # 报单价格条件
                ('Positionrate', c_double),  # 买卖方向
                ('TradingDay', c_char * 9)
                ]

class VNInstrument(Structure):
    _fields_ = [('InstrumentID', c_char * 81)]
    pass

class VNAccount(Structure):
    _fields_ = [('sUserName', c_char * 32),  #服务账号
                ('sPassword', c_char * 64)  #服务密码
                ]
    pass

class VN_CThostFtdcRspInfoField(Structure):
    _fields_ = [('ErrorID', c_int32),  # 错误代码
                ('ErrorMsg', c_char * 81)  # 错误信息
                ]
    pass

class VNStrategy(Structure):
    _fields_ = [('StrategyFileName', c_char * 300), ('StrategyType', c_int)]
    pass



class VNKlineData(Structure):
    _fields_ = [('TradingDay', c_int),
                ('klinetime', c_int),
                #('TradingDay', c_char * 9),
                #('klinetime', c_char * 9),
                ('open', c_double),
                ('high', c_double),
                ('low', c_double),
                ('close', c_double),
                ('volume', c_int32),
                ('money', c_double),
                ('open_interest', c_double),
                ('InstrumentID', c_char * 31),
                ('exchange', c_char * 31),
                ]
    pass


















